/**********************************************************************/
/* BLOCKS.C: Gestion des blocks de lignes pour l'editeur JE	      */
/**********************************************************************/

/*
    Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    Ce programme fait partie du package JE et est soumis, comme le
    reste du package JE, a la Gnu General Public License version 2
    ou superieure dont voici un extrait et dont vous pouvez lire
    la totalite en consultant le fichier COPYING.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "je.h"

extern char *memory_error;

static struct _BLOC_
	{
		FICHIER *fichier;   /* fichier dans lequel est le bloc */
		int numdeb;	    /* numero de ligne de debut du bloc */
		int numfin;	    /* numero de ligne de fin du bloc */
		LIGNE *lignedeb;    /* pointeur vers ligne de debut de bloc */
		LIGNE *lignefin;    /* pointeur vers ligne de fin du bloc */
	}
	bloc;

/**********************************************************************/
int bloc_dans_ce_fichier(FICHIER *fichier)
{
	if (fichier == bloc.fichier) /* le bloc est-il dans ce fichier ? */
	    return(1);
	else
	    return(0);
}
/**********************************************************************/
void demarquer_bloc(void)
{
	static int first = 1;	/* pour initialisation */
	LIGNE *courante;

	if (! first)
	{
		/* on demarque toutes les lignes du bloc */
		if (bloc.lignedeb != NULL)
		{
			for (courante = bloc.lignedeb; courante != bloc.lignefin->suivante; courante = courante->suivante)
				courante->debut_marque = courante->fin_marque = NOTMARKED;
		}
	}
	else
		first = 0;  /* ce n'est plus le premier passage */

	bloc.numdeb = bloc.numfin = NOTMARKED;
	bloc.lignedeb = bloc.lignefin = NULL;
	bloc.fichier = NULL;
}
/**********************************************************************/
void marquer_bloc(FICHIER *fichier)
{
	LIGNE *courante;

	/* pas le droit de marquer la premiere ou la derniere ligne */
	if ((fichier->courante->precedente == NULL) || (fichier->courante->suivante == NULL))
		return;

	/* if the line is already marked, we stop */
	/* bug corrected, Feb 09 1997, J. ALET */
	if (fichier->courante->debut_marque != NOTMARKED)
		return;

	/* debut de marquage ? */
	if (bloc.fichier == NULL)
	{
		bloc.fichier = fichier;
		bloc.lignedeb = bloc.lignefin = fichier->courante;
		bloc.numdeb = bloc.numfin = fichier->numcourante;
		fichier->courante->debut_marque = 0;	/* par exemple */
		fichier->courante->fin_marque = 1;  /* par exemple */
	}
	else
	{
		/* si c'est le meme fichier */
		/* on autorise la poursuite du marquage */
		if (fichier == bloc.fichier)
		{
			/* positionne la deuxieme borne du bloc */
			/* si debut de bloc > fin de bloc */
			/* on echange les bornes */
			if (bloc.numdeb > fichier->numcourante)
			{
				bloc.lignedeb = fichier->courante;
				bloc.numdeb = fichier->numcourante;
			}
			else
			{
				bloc.lignefin = fichier->courante;
				bloc.numfin = fichier->numcourante;
			}

			/* marque les lignes du bloc */
			for (courante = bloc.lignedeb; courante != bloc.lignefin->suivante; courante = courante->suivante)
			{
				courante->debut_marque = 0; /* par exemple */
				courante->fin_marque = 1;   /* par exemple */
			}
		}
	}
}
/**********************************************************************/
void copier_bloc(FICHIER *fichier)
{
	LIGNE *pl;
	LIGNE *courante;

	/* si un bloc est marque et que la ligne courante est hors */
	/* du bloc ou alors sur sa derniere ligne ou alors que le fichier */
	/* dans lequel on va copier est different du fichier dans lequel est le bloc */
	if ((bloc.fichier != NULL) && (fichier->courante != fichier->derniere) && ((fichier != bloc.fichier) || (fichier->numcourante < bloc.numdeb) || (fichier->numcourante >= bloc.numfin)))
	{
		for (courante = fichier->courante, pl = bloc.lignedeb; pl != bloc.lignefin->suivante; pl = pl->suivante)
		{
			courante = insere_ligne(fichier, pl->texte, courante, courante->suivante);
			if (courante == NULL)
			{
				erreur(memory_error, NULL);
				break;
			}
		}
	}

	/* le fichier a ete modifie */
	fichier->saved = 0;
}
/**********************************************************************/
void deplacer_bloc(FICHIER *fichier)
{
	copier_bloc(fichier);
	supprimer_bloc(DONTASKCONFIRMATION);

	/* le fichier a ete modifie */
	fichier->saved = 0;
}
/**********************************************************************/
void supprimer_bloc(int ask)
{
	LIGNE *avant;	    /* pointe sur la ligne avant le bloc */
	int nbl1, nbl2;	    /* nombre de lignes du bloc */

	/* si un bloc est defini et que la ligne courante n'est pas dans ce bloc */
	if ((bloc.fichier != NULL) && (bloc.fichier->courante->debut_marque == NOTMARKED))
	{
		if (((ask == ASKCONFIRMATION) && confirmation("Do you really want to remove this block ?"))
		      || (ask == DONTASKCONFIRMATION))
		{
			/* nombre de lignes du bloc */
			nbl1 = bloc.numfin - bloc.numdeb + 1;

			/* si la ligne courante est situee avant le bloc il */
			/* n'y a rien a faire */
			if (bloc.fichier->numcourante > bloc.numdeb)
			{
				/* sinon si le debut d'ecran est avant le bloc */
				/* la ligne courante est, d'apres les tests */
				/* precedents, situee plus bas que le bloc */
				/* en d'autres termes le bloc est a l'ecran */
				/* mais avant la ligne courante */
				if (bloc.fichier->numdebecr < bloc.numdeb)
				{
					/* on change le numero de ligne courante */
					/* ainsi que la ligne du curseur */
					bloc.fichier->ligne -= nbl1;
					bloc.fichier->numcourante -= nbl1;
				}
				else
				{
					/* si le debut d'ecran est dans le bloc */
					if (bloc.fichier->numdebecr <= bloc.numfin)
					{
						nbl2 = bloc.numfin - bloc.fichier->numdebecr + 1;
						bloc.fichier->numdebecr += (nbl2 - nbl1);
						bloc.fichier->ligne -= nbl2;
						bloc.fichier->numcourante -= nbl1;
						bloc.fichier->debut_ecran = bloc.lignefin->suivante;
					}
					else
					{
						/* le bloc est au dessus de l'ecran */
						/* la ligne de debut d'ecran ne change pas */
						bloc.fichier->numdebecr -= nbl1;
						bloc.fichier->numcourante -= nbl1;
					}
				}
			}

			/* supprime les lignes marquees */
			avant = bloc.lignedeb->precedente;
			while (nbl1 > 0)
			{
				supprime_ligne(bloc.fichier, avant->suivante);
				nbl1--;
			}

			/* le fichier a ete modifie */
			bloc.fichier->saved = 0;

			/* on demarque le bloc mais pas avec demarquer_bloc() */
			/* car les lignes n'existent plus */
			bloc.numdeb = bloc.numfin = NOTMARKED;
			bloc.lignedeb = bloc.lignefin = NULL;
			bloc.fichier = NULL;
		}
	}
}
/**********************************************************************/
