/**********************************************************************/
/* COULEURS.C : Definitions des couleurs pour l'Editeur JE	      */
/**********************************************************************/

/*
    Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    Ce programme fait partie du package JE et est soumis, comme le
    reste du package JE, a la Gnu General Public License version 2
    ou superieure dont voici un extrait et dont vous pouvez lire
    la totalite en consultant le fichier COPYING.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <ctype.h>
#include "je.h"

typedef struct _COULEUR_
	{
		char *couleur;
		char *color;
		int valeur;
	}
	COULEUR;

typedef struct _ATTRIBUT_
	{
		char *attribut;
		chtype valeur;
	}
	ATTRIBUT;

static COULEUR tab_couleurs[] = {
					{ "COULEUR_STATUS", "COLOR_STATUS", COULEUR_STATUS },
					{ "COULEUR_AIDE", "COLOR_HELP", COULEUR_AIDE },
					{ "COULEUR_MOTCLE", "COLOR_KEYWORD", COULEUR_MOTCLE },
					{ "COULEUR_CHAINE", "COLOR_STRING", COULEUR_CHAINE },
					{ "COULEUR_COMMENT", "COLOR_COMMENT", COULEUR_COMMENT },
					{ "COULEUR_ALERTE1", "COLOR_ALERT1", COULEUR_ALERTE1 },
					{ "COULEUR_ALERTE2", "COLOR_ALERT2", COULEUR_ALERTE2 },
					{ "COULEUR_NORMALE", "COLOR_NORMAL", COULEUR_NORMALE },
					{ "COULEUR_BLOC", "COLOR_BLOCK", COULEUR_BLOC },
					{ "COULEUR_CHAR", "COLOR_CHAR", COULEUR_CHAR },
					{ "COLOR_BLACK", "COLOR_BLACK", COLOR_BLACK },
					{ "COLOR_RED", "COLOR_RED", COLOR_RED },
					{ "COLOR_GREEN", "COLOR_GREEN", COLOR_GREEN },
					{ "COLOR_YELLOW", "COLOR_YELLOW", COLOR_YELLOW },
					{ "COLOR_BLUE", "COLOR_BLUE", COLOR_BLUE },
					{ "COLOR_MAGENTA", "COLOR_MAGENTA", COLOR_MAGENTA },
					{ "COLOR_CYAN", "COLOR_CYAN", COLOR_CYAN },
					{ "COLOR_WHITE", "COLOR_WHITE", COLOR_WHITE },
					{ NULL, NULL, -1 }
				};

static ATTRIBUT tab_attributs[] = {
					{ "A_NORMAL", A_NORMAL },
					{ "A_STANDOUT", A_STANDOUT },
					{ "A_UNDERLINE", A_UNDERLINE },
					{ "A_REVERSE", A_REVERSE },
					{ "A_BLINK", A_BLINK },
					{ "A_DIM", A_DIM },
					{ "A_BOLD", A_BOLD },
					{ NULL, 0 }
				  };

extern int tabsize;	/* taille des tabulations */
extern int backup;	/* copie de sauvegarde: YES/NO/OUI/NON */

int recherche_couleur(char **pt);
chtype recherche_attribut(char **pt);
/**********************************************************************/
void initialise_profile(char *filename)
{
	FILE *fp;
	char buffer[256];
	char *ptl;
	char *ptw;
	int paire;
	int couleur1;
	int couleur2;
	int hascolors;
	chtype attribut1;
	chtype attribut2;

	tabsize = 8;	/* valeurs par defaut */
	backup = OUI;
	language = ENGLISH;
	hascolors = NON; /* par defaut on tient compte de has_colors() */

	build_default_esckeymap(); /* default key mappings */

	/* couleurs par defaut */
	/* ce sont celles que j'utilise habituellement */
	/* il fallait bien en choisir !!! */
	init_pair(COULEUR_STATUS, COLOR_BLACK, COLOR_CYAN);
	init_pair(COULEUR_AIDE, COLOR_BLACK, COLOR_CYAN);
	init_pair(COULEUR_NORMALE, COLOR_WHITE, COLOR_BLACK);
	init_pair(COULEUR_MOTCLE, COLOR_RED, COLOR_BLACK);
	init_pair(COULEUR_CHAINE, COLOR_BLUE, COLOR_BLACK);
	init_pair(COULEUR_COMMENT, COLOR_GREEN, COLOR_BLACK);
	init_pair(COULEUR_ALERTE1, COLOR_BLACK, COLOR_CYAN);
	init_pair(COULEUR_ALERTE2, COLOR_BLACK, COLOR_CYAN);
	init_pair(COULEUR_BLOC, COLOR_WHITE, COLOR_BLUE);
	init_pair(COULEUR_CHAR, COLOR_MAGENTA, COLOR_BLACK);

	couleur[COULEUR_STATUS] = COLOR_PAIR(COULEUR_STATUS);
	couleur[COULEUR_AIDE] = COLOR_PAIR(COULEUR_AIDE);
	couleur[COULEUR_NORMALE] = COLOR_PAIR(COULEUR_NORMALE);
	couleur[COULEUR_MOTCLE] = COLOR_PAIR(COULEUR_MOTCLE) | A_BOLD;
	couleur[COULEUR_CHAINE] = COLOR_PAIR(COULEUR_CHAINE) | A_BOLD;
	couleur[COULEUR_COMMENT] = COLOR_PAIR(COULEUR_COMMENT);
	couleur[COULEUR_ALERTE1] = COLOR_PAIR(COULEUR_ALERTE1);
	couleur[COULEUR_ALERTE2] = COLOR_PAIR(COULEUR_ALERTE2) | A_REVERSE | A_BLINK;
	couleur[COULEUR_BLOC] = COLOR_PAIR(COULEUR_BLOC);
	couleur[COULEUR_CHAR] = COLOR_PAIR(COULEUR_CHAR) | A_BOLD;

	/* on essaie d'ouvrir le fichier des preferences */
	fp = fopen(filename, "rt");
	if (fp != NULL)
	{
		/* on lit les couleurs dans le fichier */
		while ((ptl = fgets(buffer, 255, fp)) != NULL)
		{
			/* passe la ligne lue en majuscules */
			for (ptw = ptl; *ptw; ptw++)
				*ptw = (char)toupper((unsigned int)(unsigned char)*ptw);

			if (! strncmp(ptl, "ESC_", 4))
			{
				if (isalpha(*(ptl + 4)) && (*(ptl + 5) == ':'))
				{
					*(ptl + 5) = '\0';
					ptw = ptl + 6;
					while (*ptw && (! isspace(*ptw)))
						ptw++;
					*ptw = '\0';
					mapesckeytofunction(ptl, ptl + 6);
				}
			}
			else if (! strncmp(ptl, "TABSIZE:", 8))
				tabsize = atoi(ptl + 8);
			else if (! strncmp(ptl, "BACKUP:", 7))
			{
				/* ce double test est bien inutile, mais enfin */
				if (! strncmp(ptl + 7, "NON", 3) || ! strncmp(ptl + 7, "NO", 2))
					backup = NON;
			}
			else if (! strncmp(ptl, "COLORS:", 7))
			{
				if (! strncmp(ptl + 7, "YES", 3) || ! strncmp(ptl + 7, "OUI", 3))
					hascolors = OUI;
			}
			else if (! strncmp(ptl, "LANGUAGE:", 9))
			{
				if (! strncmp(ptl + 9, "FRENCH", 6) || ! strncmp(ptl + 9, "FRANCAIS", 8))
					language = FRENCH;
			}
			else
			{
				paire = recherche_couleur(&ptl);
				if (paire)
				{
					couleur1 = recherche_couleur(&ptl);
					couleur2 = recherche_couleur(&ptl);
					attribut1 = recherche_attribut(&ptl);
					attribut2 = recherche_attribut(&ptl);

					init_pair(paire, couleur1, couleur2);
					couleur[paire] = attribut1 | attribut2 | COLOR_PAIR(paire);
				}
			}
		}
		fclose(fp);
	}

	/* si le terminal n'accepte pas les couleurs on force */
	/* des valeurs pas trop craignos */
	if ((hascolors == NON) && (has_colors() == FALSE))
	{
		/* le terminal n'accepte pas les couleurs */
		couleur[COULEUR_STATUS] = A_REVERSE;
		couleur[COULEUR_AIDE] = A_REVERSE;
		couleur[COULEUR_NORMALE] = A_NORMAL;
		couleur[COULEUR_MOTCLE] = A_BOLD;
		couleur[COULEUR_CHAINE] = A_BOLD;
		couleur[COULEUR_COMMENT] = A_REVERSE;
		couleur[COULEUR_ALERTE1] = A_REVERSE;
		couleur[COULEUR_ALERTE2] = A_NORMAL;
		couleur[COULEUR_BLOC] = A_REVERSE | A_BLINK;
		couleur[COULEUR_CHAR] = A_BOLD;
	}
}
/**********************************************************************/
int recherche_couleur(char **pt)
{
	COULEUR *ptc;
	int trouve = 0;
	int lg;

	for (ptc = tab_couleurs; ptc->couleur != NULL; ptc++)
	{
		lg = strlen(ptc->couleur);
		if (strncmp(ptc->couleur, *pt, lg))
		{
			lg = strlen(ptc->color);
			if (strncmp(ptc->color, *pt, lg))
				lg = -1;
		}
		if (lg != -1)
		{
			trouve = ptc->valeur;
			(*pt) += lg;
			while (**pt && (**pt != ':'))
				(*pt)++;
			if (**pt)
				(*pt)++;	   /* on saute les ':' */

			/* on pointe vers le prochain mot */
			while (**pt && ((**pt == ' ') || (**pt == '\t')))
				(*pt)++;
			break;
		}
	}
	return(trouve);
}
/**********************************************************************/
chtype recherche_attribut(char **pt)
{
	ATTRIBUT *pta;
	chtype trouve = A_NORMAL;

	for (pta = tab_attributs; pta->attribut != NULL; pta++)
	{
		if (! strncmp(pta->attribut, *pt, strlen(pta->attribut)))
		{
			trouve = pta->valeur;
			(*pt) += strlen(pta->attribut);
			while (**pt && (**pt != ':'))
				(*pt)++;
			if (**pt)
				(*pt)++;	/* on saute les ':' */
			while (**pt && ((**pt == ' ') || (**pt == '\t')))
				(*pt)++;
			break;
		}
	}
	return(trouve);
}
/**********************************************************************/
