/**********************************************************************/
/* ECRAN.C : Tous les affichages et les saisies bas niveau de JE      */
/**********************************************************************/

/*
    Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    Ce programme fait partie du package JE et est soumis, comme le
    reste du package JE, a la Gnu General Public License version 2
    ou superieure dont voici un extrait et dont vous pouvez lire
    la totalite en consultant le fichier COPYING.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include "je.h"
#include "formulr.h"

extern PTFORMU formmacro;
extern char vmacroname[66];

static int oldcursor = 0;	/* ancienne valeur du curseur */
static int firstcursor = 0;	/* premier curseur au debut de l'appli */
static int rafraichir = 1;	/* par defaut on rafraichit */

/**********************************************************************/
void debute_ecran(void)
{
	initscr();		/* initialise curses */
	start_color();		/* initialise la gestion des couleurs */

	cbreak();		/* interdit la bufferisation et le traitement de certains caracteres */
	noecho();		/* n'affiche pas les caracteres lus par getch */
	nonl();
	keypad(stdscr, TRUE);	/* autorise les touches speciales du terminal a etre renvoyees telles quelles (ex: F1..F10) */
	meta(stdscr, TRUE);	/* autorise le terminal a renvoyer des valeurs de 8 bits au lieu de 7 */
	nodelay(stdscr, FALSE); /* oblige getch a attendre qu'une touche soit tapee avant de revenir */
	idlok(stdscr, FALSE);	/* interdit les insertions/suppressions de ligne */
	scrollok(stdscr, FALSE); /* interdit le scrolling automatique */

	firstcursor = allume_curseur(0);	/* sauve valeur du curseur et l'eteint */
	oldcursor = firstcursor;

	XMAX = (COLS - 1);
	YMAX = (LINES - 2);
}
/**********************************************************************/
void termine_ecran(void)
{
	clr_scr();			/* efface l'ecran */
	allume_curseur(firstcursor);	/* remet le curseur d'origine */
	endwin();			/* termine l'utilisation de curses */
}
/**********************************************************************/
int allume_curseur(int type)
{
	oldcursor = curs_set(type);
	if (oldcursor == ERR)
		oldcursor = 1;
	return(oldcursor);
}
/**********************************************************************/
void eteint_curseur(void)
{
	oldcursor = curs_set(0);
	if (oldcursor == ERR)
		oldcursor = 1;
}
/**********************************************************************/
void restaure_curseur(void)
{
	oldcursor = curs_set(oldcursor);
	if (oldcursor == ERR)
		oldcursor = 1;
}
/**********************************************************************/
void clr_scr(void)
{
	clear();	/* efface l'ecran */
	refresh();	/* validation */
}
/**********************************************************************/
int fread_key(void)
{
	static FILE *fmacro = NULL;	/* fichier macro */
	static int indic = 0;		/* indique si on saisit le nom de la macro */
	int key;			/* touche appuyee */
	char cmd[256];			/* ligne de commande */

	/* si on est en train de saisir le nom de la macro */
	/* on ne permet pas l'execution ou l'enregistrement de l'une d'entre elles */
	if (indic)
	{
		flushinp();
		return(wgetch(stdscr));
	}

	if (macro == PLAYING)
	{
		/* on lit une touche depuis le fichier */
		if (fread(&key, sizeof(key), 1, fmacro) != 1)
		{
			/* s'il est fini */
			fclose(fmacro);
			fmacro = NULL;
			macro = NOMACRO;

			/* on lit la prochaine touche au clavier */
			goto lire_clavier; /* pas elegant !!! */
		}
	}
	else
	{
lire_clavier:
		flushinp();
		key = wgetch(stdscr);
		if (key == Ctrl_R)
		{
			if (macro != RECORDING)
			{
				/* on debute l'enregistrement */
				fmacro = fopen(MACROFILE, "w+b");
				if (fmacro != NULL)
					macro = RECORDING;
				else
					macro = NOMACRO;
			}
			else
			{
				/* si on arrete l'enregistrement */
				fclose(fmacro);

				indic = 1; /* evite d'etre reentrant */
				raz_zones(formmacro);
				if (lire_formulaire_simple(formmacro) == OKBUTTON)
				{
					/* copie la macro par defaut sous son nouveau nom */
					if (*vmacroname)
					{
						sprintf(cmd, "cp %s %s >/dev/null 2>/dev/null", MACROFILE, vmacroname);
						system(cmd);
					}
				}
				indic = 0; /* reautorise re-entrance */
				fmacro = NULL;
				macro = RECORDED;
			}
		}
		else if (key == Ctrl_P) /* on veut jouer la macro */
		{
			/* si macro pas terminee */
			if (fmacro != NULL)
			{
				fclose(fmacro);
				macro = RECORDED;
			}

			/* on reouvre le fichier pour le lire */
			fmacro = fopen(MACROFILE, "r+b");
			if (fmacro != NULL)
				macro = PLAYING;
			else
				macro = NOMACRO;	/* erreur */
		}
		else if (macro == RECORDING) /* ecrit la touche dans le fichier */
			fwrite(&key, sizeof(key), 1, fmacro);
	}
	return(key);
}
/**********************************************************************/
int set_refresh(int on_off)
{
	int old;

	old = rafraichir;
	rafraichir = on_off;
	return(old);
}
/**********************************************************************/
int get_refresh(void)
{
	return(rafraichir);
}
/**********************************************************************/
int get_txt(int x, int y, int xmax, int ymax, chtype *dest)
{
	int sauvex;		/* sauve abscisse */
	int sauvey;		/* sauve ordonnee */
	int lg;			/* longueur */

	xmax = min(xmax, (COLS - 1));
	ymax = min(ymax, (LINES - 1));
	if ((x > xmax) || (x < 0) || (y > ymax) || (y < 0))
		return(-1);
	else
	{
		getyx(stdscr, sauvey, sauvex);	/* sauve la position du curseur */
		lg = xmax - x + 1;
		while (y <= ymax)
		{
			for (lg = x; lg <= xmax; lg++)
			{
				move(y, lg);
				*dest++ = inch();
			}
			y++;
		}
		move(sauvey, sauvex);		/* restitue la position du curseur */

		if (rafraichir)
			refresh();	/* n'est ce pas inutile ? */

		return(0);
	}
}
/**********************************************************************/
int put_txt(int x, int y, int xmax, int ymax, chtype *src)
{
	int sauvex;		/* sauve abscisse */
	int sauvey;		/* sauve ordonnee */
	int lg;			/* longueur */

	xmax = min(xmax, (COLS - 1));
	ymax = min(ymax, (LINES - 1));
	if ((x > xmax) || (x < 0) || (y > ymax) || (y < 0))
		return(-1);
	else
	{
		getyx(stdscr, sauvey, sauvex);	/* sauve la position du curseur */
		lg = xmax - x + 1;
		while (y <= ymax)
		{
#ifndef __NCURSES_NOT_PRESENT__ /* il y a NCURSES */
			mvwaddchnstr(stdscr, y, x, src, lg); /* copie les lg chtype a partir de src */
			src += lg;
#else
			for (lg = x; lg <= xmax; lg++)
			{
				move(y, lg);
				addch(*src);
				src++;
			}
#endif /* __NCURSES_NOT_PRESENT__ */
			y++;
		}
		move(sauvey, sauvex);		/* restitue la position du curseur */

		if (rafraichir)
			refresh();

		return(0);
	}
}
/**********************************************************************/
void scrollup(int ymin, int ymax)
{
	chtype *sauve;	/* sauve pointeur sur la premiere ligne */
	chtype *ptfin;	/* pointeur sur fin de ligne */

#ifndef __NCURSES_NOT_PRESENT__ /* il y a NCURSES */
	struct ldat *pl; /* pointeur vers ligne courante; */
	struct ldat *pf; /* pointeur vers derniere ligne */

	pl = &stdscr->_line[ymin];
	pf = &stdscr->_line[ymax];
	sauve = pl->text;
	while (pl != pf)
	{
		pl->text = (pl + 1)->text;
		pl->firstchar = 0;
		pl->lastchar = XMAX;
		pl++;
	}
	pl->text = sauve;
	pl->firstchar = 0;
	pl->lastchar = XMAX;
#else
	int l;

	sauve = stdscr->_y[ymin];
	for (l = ymin; l < ymax; l++)
	{
		stdscr->_y[l] = stdscr->_y[l + 1];
		stdscr->_firstch[l] = (short)0;
		stdscr->_lastch[l] = (short)XMAX;
	}
	stdscr->_y[l] = sauve;
	stdscr->_firstch[l] = (short)0;
	stdscr->_lastch[l] = (short)XMAX;
#endif	/* __NCURSES_NOT_PRESENT__ */

	ptfin = sauve + XMAX;
	while (sauve <= ptfin)
		*sauve++ = (chtype)' ' | couleur[COULEUR_NORMALE];

	if (rafraichir)
		refresh();
}
/**********************************************************************/
void scrolldown(int ymin, int ymax)
{
	chtype *sauve;	/* sauve pointeur sur la derniere ligne */
	chtype *ptfin;	/* pointeur sur fin de ligne */

#ifndef __NCURSES_NOT_PRESENT__ /* il y a NCURSES */
	struct ldat *pl; /* pointeur vers ligne courante; */
	struct ldat *pd; /* pointeur vers premiere ligne */

	pl = &stdscr->_line[ymax];
	pd = &stdscr->_line[ymin];
	sauve = pl->text;
	while (pl != pd)
	{
		pl->text = (pl - 1)->text;
		pl->firstchar = 0;
		pl->lastchar = XMAX;
		pl--;
	}
	pl->text = sauve;
	pl->firstchar = 0;
	pl->lastchar = XMAX;
#else
	int l;

	sauve = stdscr->_y[ymax];
	for (l = ymax; l > ymin; l--)
	{
		stdscr->_y[l] = stdscr->_y[l - 1];
		stdscr->_firstch[l] = (short)0;
		stdscr->_lastch[l] = (short)XMAX;
	}
	stdscr->_y[l] = sauve;
	stdscr->_firstch[l] = (short)0;
	stdscr->_lastch[l] = (short)XMAX;
#endif	/* __NCURSES_NOT_PRESENT__ */

	ptfin = sauve + XMAX;
	while (sauve <= ptfin)
		*sauve++ = (chtype)' ' | couleur[COULEUR_NORMALE];

	if (rafraichir)
		refresh();
}
/**********************************************************************/
void put_statligne(int x, int y, char *texte, chtype attrib)
{
	chtype buffer[256];	   /* buffer pour construire ecran */
	chtype *ptw;		   /* pointeur sur ce buffer */

	/* cree la ligne et la complete avec des espaces */
	ptw = buffer;
	if (texte != NULL)
	{
		while (*texte)
			*ptw++ = (chtype)(unsigned char)*texte++ | attrib;
	}
	while (ptw < buffer + COLS)
		*ptw++ = (chtype)' ' | attrib;

	put_txt(x, y, (COLS - 1), y, buffer);
}
/**********************************************************************/
