/**********************************************************************/
/* ESCAPE.C: Escape keys handling functions			      */
/**********************************************************************/

/*
    Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    Ce programme fait partie du package JE et est soumis, comme le
    reste du package JE, a la Gnu General Public License version 2
    ou superieure dont voici un extrait et dont vous pouvez lire
    la totalite en consultant le fichier COPYING.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include "je.h"

	/* just add new functions to the list below */
	/* for now, add only int functions with no parameter */
static FUNCMAP functable[] =
	{
		{ "ESCAPE_HELP", &escape_help },
		{ NULL, NULL }
	};

	/* for now, we just allow ESC_x where x is an alphabetical key */
	/* but we can modify this later */
#define ESCKEY_INDEX(k)		(int)(k - 'A')

static ESCKEYMAP esckeytable[ESCKEY_INDEX('Z') + 2] =
	{
		{ 'A', "ESC_A", NULL },
		{ 'B', "ESC_B", NULL },
		{ 'C', "ESC_C", NULL },
		{ 'D', "ESC_D", NULL },
		{ 'E', "ESC_E", NULL },
		{ 'F', "ESC_F", NULL },
		{ 'G', "ESC_G", NULL },
		{ 'H', "ESC_H", NULL },
		{ 'I', "ESC_I", NULL },
		{ 'J', "ESC_J", NULL },
		{ 'K', "ESC_K", NULL },
		{ 'L', "ESC_L", NULL },
		{ 'M', "ESC_M", NULL },
		{ 'N', "ESC_N", NULL },
		{ 'O', "ESC_O", NULL },
		{ 'P', "ESC_P", NULL },
		{ 'Q', "ESC_Q", NULL },
		{ 'R', "ESC_R", NULL },
		{ 'S', "ESC_S", NULL },
		{ 'T', "ESC_T", NULL },
		{ 'U', "ESC_U", NULL },
		{ 'V', "ESC_V", NULL },
		{ 'W', "ESC_W", NULL },
		{ 'X', "ESC_X", NULL },
		{ 'Y', "ESC_Y", NULL },
		{ 'Z', "ESC_Z", NULL },
		{ 0, NULL, NULL }
	};

/**********************************************************************/
int handle_escape_key(void)
{
	int key;

	if (language == ENGLISH)
		put_statligne(0, (LINES - 1), "ESC: Read the help file to know which functions are available", couleur[COULEUR_AIDE]);
	else
		put_statligne(0, (LINES - 1), "ESC: Lisez le fichier d'aide pour connaitre les fonctions disponibles", couleur[COULEUR_AIDE]);

	key = fread_key();	/* read a key from keyboard */
	if (key == ESCAPE)
		return(0);	/* nothing to do */
	else if (isalpha(key))	/* for now we only allow alphabetical ESC keys */
	{
		key = toupper(key);
		if (esckeytable[ESCKEY_INDEX(key)].func != NULL)
		{
			/* handle the ESC_x key */
			return(esckeytable[ESCKEY_INDEX(key)].func->function());
		}
	}
	if (language == ENGLISH)
		alerte(1, "Undefined ESCAPE function", "OK");
	else
		alerte(1, "Fonction ESCAPE indefinie", "OK");
	return(0);	/* nothing to do */
}
/**********************************************************************/
void mapesckeytofunction(char *esckeyname, char *funcname)
{
	ESCKEYMAP *pk;

	for (pk = esckeytable; pk->esckeyname != NULL; pk++)
	{
		if (! strcmp(pk->esckeyname, esckeyname))
		{
			pk->func = findescfunction(funcname);
			break;
		}
	}
}
/**********************************************************************/
FUNCMAP *findescfunction(char *funcname)
{
	FUNCMAP *p;

	for (p = functable; p->funcname != NULL; p++)
	{
		if (! strcmp(p->funcname, funcname))
			return(p);
	}
	return(NULL);
}
/**********************************************************************/
void build_default_esckeymap(void)
{
	mapesckeytofunction("ESC_H", "ESCAPE_HELP");
}
/**********************************************************************/
int escape_help(void)
{
	char *fname;

	fname = getenv("JEHELP");
	if (fname == NULL)	/* not defined */
		fname = "./help.je"; /* default file to load */

	edite_autre_fichier(fname);
	return(FICHIERSUIVANT); /* we must return this value */
}
/**********************************************************************/
