/**********************************************************************/
/* JE.C	   : Editeur de texte facile d'utilisation		      */
/**********************************************************************/

/*
    Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    Ce programme fait partie du package JE et est soumis, comme le
    reste du package JE, a la Gnu General Public License version 2
    ou superieure dont voici un extrait et dont vous pouvez lire
    la totalite en consultant le fichier COPYING.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define _JE_TABCOUL_

#include <sys/types.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/wait.h>
#include "je.h"
#include "formulr.h"

#if !defined(WIFEXITED)
#define WIFEXITED(a)	(! (a & 0x00ff))
#endif /* WIFEXITED */

static char *usage0 = "\nJE v1.4 (C) 1996, 1997 Free Software Foundation\nusage:  je  file1  file2  ...  fileN\n";
static char *usage1 = "JE comes with ABSOLUTELY NO WARRANTY\n";
static char *usage2 = "This is free software, and you are welcome to redistribute it under certain\n";
static char *usage3 = "conditions; refer to the Gnu General Public License for details.\n";
static char *copyright = "\nPlease e-mail bugs to: jalet@nat.fr\n\n";

typedef struct _FILELIST_
	{
		FICHIER *fichier;
		struct _FILELIST_ *suivant;
	}
	FILELIST;

static char buftmp[SZBUF + 2];	/* buffer de travail */
static FILELIST *debut_liste;	/* pointeur vers debut de la liste des fichiers */

char *memory_error = "Memory error";	/* valeur par defaut */
int language;
int insert_mode;		/* mode d'insertion/recouvrement */
int macro = NOMACRO;		/* etat de la macro eventuelle */
int XMAX;			/* abscisse maxi du texte */
int YMAX;			/* ordonnee maxi du texte */
int tabsize;			/* valeur des tabulations */
int backup;			/* indique si oui ou non on veut une copie de sauvegarde */
chtype couleur[NBCOULEURS];	/* table des couleurs */

extern PTFORMU fcherche;
extern PTFORMU fcmdline;
extern char vcmdline[66];
extern char vachercher[42];
extern char vremplacer[42];
extern char vignorer[2];

LIGNE *bas(FICHIER *fichier);
LIGNE *haut(FICHIER *fichier);
void droite(FICHIER *fichier, char *buf, int affiche, int end);
void gauche(FICHIER *fichier, char *buf, int affiche, int home);
int saisir_ligne(FICHIER *fichier);
void strip(FILE *fpo, char *buffer);
void expand(char *texte, char *buffer);
int chargement(FILE *fpi, FICHIER *fichier);
void dechargement(FILE *fpo, FICHIER *fichier);
int edition(FICHIER *fichier);
void affiche_ecran(FICHIER *fichier, int decalage, LIGNE *ignore);
int realloue_texte(LIGNE *plig, char *texte);
void insertion_ligne(FICHIER *fichier);
void destruction_ligne(FICHIER *fichier);
void remonter_ligne(FICHIER *fichier);
void descendre_ligne(FICHIER *fichier);
void descendre_page(FICHIER *fichier);
void remonter_page(FICHIER *fichier);
void appelle_shell(void);
char *inserespace(char *chaine, int position, int nombre);
int create_forms(void);
void delete_forms(void);
int execute_commande(FICHIER *fichier);
void chercher_remplacer(FICHIER *fichier);
char *cherche(char *, char *);
int sauvegarder(FICHIER *fichier);
void liberation_memoire(FICHIER *fichier);
int initialise_fichier(FICHIER *fichier);
FILELIST *ajoute_fichier_dans_liste(FICHIER *fichier);
FILELIST *supprime_fichier_de_liste(FILELIST *courant);
void goto_line(FICHIER *fichier, int num);
void top_of_file(FICHIER *fichier);
/**********************************************************************/
int main(int argc, char **argv)
{
	int retcode = 0;	/* code de retour */
	char *profilename;	/* nom du fichier profile */
	char *mess;		/* message */

	if ((argc < 2) || ((argc == 2) && ((! strcmp(argv[1], "-h")) || (! strcmp(argv[1], "-?")) || (! strcmp(argv[1], "--help")))))
	{
		fprintf(stdout, usage0);
		fprintf(stdout, usage1);
		fprintf(stdout, usage2);
		fprintf(stdout, usage3);
		fprintf(stdout, copyright);
		exit(0);
	}

	debute_ecran();

	profilename = (char *)getenv("JEPROFILE");
	if (profilename == NULL)
		profilename = "profile.je";
	initialise_profile(profilename);

	if (language == ENGLISH)
		mess = "Please wait...";
	else
	{
		mess = "Veuillez patienter...";
		memory_error = "Erreur de memoire";
	}
	move(0, 0);
	printw("%s", mess);
	refresh();

	if (! create_forms())
	{
		/* on passe en mode insertion et on efface l'ecran */
		insert_mode = INSERT;
		allume_curseur(1); /* petit curseur */

		/* on commence sans bloc */
		demarquer_bloc();

		/* et avec une liste vide */
		debut_liste = NULL;
		while ((--argc) && (! retcode))
			retcode = edite_autre_fichier(argv[argc]);

		/* efface l'ecran */
		clr_scr();

		/* si pas d'erreur on a la liste circulaire des fichiers et */
		/* ils sont tous charges en memoire */
		if (! retcode)
		{
			/* si la liste des fichiers n'est pas vide */
			if (debut_liste != NULL) /* ce test est normalement en trop ! */
			{
				/* debut_liste pointe sur le dernier fichier */
				/* on le fait donc pointer sur le premier */
				debut_liste = debut_liste->suivant;

				/* tant que la liste des fichiers n'est pas vide */
				while (debut_liste != NULL)
				{
					retcode = edition(debut_liste->fichier);
					if (retcode == QUITTE)
					{
						liberation_memoire(debut_liste->fichier);
						debut_liste = supprime_fichier_de_liste(debut_liste);
					}
					else if (retcode == FICHIERSUIVANT)
						debut_liste = debut_liste->suivant;
					else
					{
						if (language == ENGLISH)
							erreur("Unexpected return to main loop encountered", NULL);
						else
							erreur("Retour inattendu en boucle principale", NULL);
					}
				}
			}
		}
	}
	delete_forms();

	termine_ecran();

	exit(retcode);
}
/**********************************************************************/
int edite_autre_fichier(char *filename)
{
	int retcode = -1;		/* code de retour */
	FICHIER *fichier_courant;	/* pointeur sur fichier courant */

	/* alloue memoire pour fichier courant */
	fichier_courant = (FICHIER *)malloc(sizeof(FICHIER));
	if (fichier_courant != NULL)
	{
		/* on l'ajoute dans la liste */
		debut_liste = ajoute_fichier_dans_liste(fichier_courant);
		if (debut_liste != NULL)
		{
			fichier_courant->filename = filename;
			retcode = initialise_fichier(fichier_courant);
		}
		else
			erreur(memory_error, NULL);
	}
	else
		erreur(memory_error, NULL);

	return(retcode);
}
/**********************************************************************/
FILELIST *supprime_fichier_de_liste(FILELIST *courant)
{
	FILELIST *precedent;

	if (courant->suivant == courant)	/* dernier element de la liste ? */
	{
		/* on le libere */
		free(courant);

		/* et on indique que la fin de la liste est atteinte */
		return(NULL);
	}
	else
	{
		/* cherche l'element precedent le courant dans la liste */
		for (precedent = courant; precedent->suivant != courant; precedent = precedent->suivant) ;

		/* on casse la liaison vers le courant */
		precedent->suivant = courant->suivant;

		/* on libere l'entree courante */
		free(courant);

		/* en on renvoie un pointeur vers l'element suivant dans la liste */
		return(precedent->suivant);
	}
}
/**********************************************************************/
FILELIST *ajoute_fichier_dans_liste(FICHIER *fichier)
{
	FILELIST *nouveau_debut;

	nouveau_debut = (FILELIST *)malloc(sizeof(FILELIST));
	if (nouveau_debut != NULL)
	{
		/* si liste non vide */
		if (debut_liste != NULL)
		{
			nouveau_debut->fichier = fichier;
			nouveau_debut->suivant = debut_liste->suivant;
			debut_liste->suivant = nouveau_debut;
			return(debut_liste);
		}
		else
		{
			nouveau_debut->fichier = fichier;
			nouveau_debut->suivant = nouveau_debut;
		}
	}
	return(nouveau_debut);
}
/**********************************************************************/
void liberation_memoire(FICHIER *fichier)
{
	LIGNE *courante;	/* ptr sur ligne courante */
	LIGNE *suivante;	/* ptr sur ligne suivante */

	/* si le bloc est dans le fichier que l'on libere  */
	/* alors on demarque le bloc pour ne pas avoir */
	/* un bloc pointant vers des lignes inexistantes */
	if (bloc_dans_ce_fichier(fichier))
		demarquer_bloc();

	for (courante = fichier->premiere->suivante; courante != fichier->derniere; courante = suivante)
	{
		/* libere la ligne courante */
		suivante = courante->suivante;	/* sauve pointeur vers suivante */
		if (courante->texte != NULL)
			free(courante->texte);	/* libere texte de la ligne */
		free(courante);			/* libere la ligne elle meme */
	}
	free(fichier->premiere);
	free(fichier->derniere);
	free(fichier);
}
/**********************************************************************/
int sauvegarder(FICHIER *fichier)
{
	char tmp[256];		/* pour nom de fichier temporaire */
	FILE *fpo;		/* id fichier temporaire */
	FILE *fpi;		/* id du fichier que l'on edite */
	char *ptl;		/* pointeur vers ligne */

	/* si le fichier vient d'etre sauve on ne fait rien */
	if (fichier->saved)
		return(0);

	/* affiche le message comme quoi la sauvegarde est en cours */
	if (language == ENGLISH)
		sprintf(tmp, "Saving:  %s", fichier->filename);
	else
		sprintf(tmp, "Sauvegarder:  %s", fichier->filename);
	put_statligne(0, (LINES - 1), tmp, couleur[COULEUR_AIDE]);

	/* si copie de sauvegarde on essaie de l'effectuer */
	if (backup == OUI)
	{
		sprintf(tmp, "cp %s %s.bak 1>/dev/null 2>/dev/null", fichier->filename, fichier->filename);
		system(tmp);
	}

	/* cree un nom de fichier temporaire */
	tmpnam(tmp);

	/* maintenant on cree le fichier */
	fpo = fopen(tmp, "w+t");
	if (fpo == NULL)
	{
		if (language == ENGLISH)
			erreur("Error while creating %s", tmp);
		else
			erreur("Erreur en creant %s", tmp);
		return(-1);
	}
	else
	{
		/* on sauve le texte dans le fichier */
		dechargement(fpo, fichier);

		/* on se replace en debut de fichier temporaire */
		rewind(fpo);

		/* on ne peut pas renommer le fichier temporaire */
		/* sinon on prend les droits de l'utilisateur actuel */
		/* il faut donc le copier dans le fichier que l'on edite */
		/* apres l'avoir remis a zero (wt) */
		fpi = fopen(fichier->filename, "w+t");
		if (fpi != NULL)
		{
			while ((ptl = fgets(buftmp, SZBUF, fpo)) != NULL)
				fputs(buftmp, fpi);

			/* on ferme le fichier d'entree */
			fclose(fpi);

			/* on ferme le fichier temporaire */
			fclose(fpo);

			/* et comme tout est ok: on l'efface */
			if (unlink(tmp))
			{
				if (language == ENGLISH)
					erreur("Error while unlinking %s", tmp);
				else
					erreur("Erreur en effacant %s", tmp);
			}
			/* le fichier a ete sauve */
			fichier->saved = 1;

			return(0);
		}
		else
		{
			/* on ferme le fichier temporaire */
			/* mais on ne l'efface pas !!! */
			fclose(fpo);

			if (language == ENGLISH)
				erreur("Error while accessing %s", fichier->filename);
			else
				erreur("Erreur en accedant a %s", fichier->filename);
			return(-1);
		}
	}
}
/**********************************************************************/
int initialise_fichier(FICHIER *fichier)
{
	int retcode = 0;	/* par defaut pas d'erreur */
	FILE *fpi;		/* pointeur sur fichier d'entree */


	fichier->first = 1;	/* premier passage dans ce fichier */
	fichier->saved = 1;	/* il n'y a pas encore eu de modifications */

	/* si c'est un fichier de type C/C++ */
	/* on colorise, sinon non */
	if (cfamily(fichier->filename))
		fichier->affiche_ligne = put_ligne;
	else
		fichier->affiche_ligne = print_ligne;

	/* on ouvre le fichier mais on ne tient pas compte de l'echec */
	/* car on peut vouloir editer un fichier qui n'existe pas encore */
	fpi = fopen(fichier->filename, "rt");

	/* alloue la premiere et la derniere ligne */
	fichier->premiere = (LIGNE *)malloc(sizeof(LIGNE));
	fichier->derniere = (LIGNE *)malloc(sizeof(LIGNE));
	if ((fichier->premiere == NULL) || (fichier->derniere == NULL))
	{
		erreur(memory_error, NULL);
		retcode = -1;
	}
	else
	{
		fichier->premiere->suivante = fichier->derniere;
		fichier->derniere->precedente = fichier->premiere;
		fichier->premiere->precedente = fichier->derniere->suivante = NULL;
		if (language == ENGLISH)
		{
			fichier->premiere->texte = "==== TOP OF FILE ====";
			fichier->derniere->texte = "==== END OF FILE ====";
		}
		else
		{
			fichier->premiere->texte = "==== DEBUT DE FICHIER ====";
			fichier->derniere->texte = "====  FIN DE FICHIER  ====";
		}
		fichier->premiere->debut_marque = fichier->derniere->debut_marque = NOTMARKED;
		fichier->premiere->fin_marque = fichier->derniere->fin_marque = NOTMARKED;

		if (chargement(fpi, fichier))
		{
			if (language == ENGLISH)
				erreur("Error while reading %s", fichier->filename);
			else
				erreur("Erreur en lisant %s", fichier->filename);
			retcode = -1;
		}
		else
		{
			top_of_file(fichier);
			retcode = 0;	/* chargement OK */
		}
		if (fpi != NULL)
			fclose(fpi);	/* ferme le fichier d'entree */
	}
	return(retcode);
}
/**********************************************************************/
void top_of_file(FICHIER *fichier)
{
	fichier->debut_ecran = fichier->premiere;
	fichier->courante = fichier->premiere->suivante;
	fichier->numcourante = 1;
	fichier->numdebecr = 0;
	fichier->ligne = YMIN + 1;
	fichier->colonne = 0;
	fichier->premier_caractere = 0;
}
/**********************************************************************/
int edition(FICHIER *fichier)
{
	int quitte;		/* indicateur de sortie */
	int saisie;		/* indic de sortie de saisie de ligne */

	affiche_ecran(fichier, 0, NULL);
	quitte = 0;
	while (! quitte)
	{
		saisie = saisir_ligne(fichier);
		switch (saisie)
		{
			case COMMANDE:
				quitte = execute_commande(fichier);
				break;

			case CHERCHE:
				chercher_remplacer(fichier);
				break;

			case QUITTE:
				quitte = QUITTE;
				break;

			case FICHIERSUIVANT:
				quitte = FICHIERSUIVANT;
				break;

			case SAUVE:
				sauvegarder(fichier);
				break;

			case SHELL:
				appelle_shell();
				break;

			case REMONTE_PAGE:
				remonter_page(fichier);
				break;

			case DESCEND_PAGE:
				descendre_page(fichier);
				break;

			case REMONTE_LIGNE:
				remonter_ligne(fichier);
				break;

			case DESCEND_LIGNE:
				descendre_ligne(fichier);
				break;

			case INSERE_LIGNE:
				insertion_ligne(fichier);
				fichier->saved = 0; /* le fichier a ete modifie */
				break;

			case DETRUIT_LIGNE:
				destruction_ligne(fichier);
				fichier->saved = 0; /* le fichier a ete modifie */
				break;

			case DEMARQUER_BLOC:
				demarquer_bloc();
				affiche_ecran(fichier, 0, NULL);
				break;

			case MARQUER_BLOC:
				marquer_bloc(fichier);
				affiche_ecran(fichier, 0, NULL);
				break;

			case COPIER_BLOC:
				copier_bloc(fichier);
				affiche_ecran(fichier, 0, NULL);
				break;

			case DEPLACER_BLOC:
				deplacer_bloc(fichier);
				affiche_ecran(fichier, 0, NULL);
				break;

			case SUPPRIMER_BLOC:
				supprimer_bloc(ASKCONFIRMATION);
				affiche_ecran(fichier, 0, NULL);
				break;

			default:
				beep();
				if (language == ENGLISH)
					erreur("Unknown function", NULL);
				else
					erreur("Fonction inconnue", NULL);
				break;
		}
	}
	return(quitte);
}
/**********************************************************************/
void chercher_remplacer(FICHIER *fichier)
{
	static int first = 1;	/* premier passage */
	char *ptrouve;		/* pointeur sur code trouve */
	char *ptr;		/* pointeur pour la recherche */
	int touche;		/* code de retour de la question */
	int fini;		/* indique la fin de la recherche */
	char buf[SZBUF + 2];	/* pour remplacements */
	char *pts;		/* pointeur source pour remplacement */
	char *ptd;		/* pointeur destination pour remplacement */
	int diff;		/* offset du motif trouve dans la ligne */
	int tous;		/* pour tout remplacer */
	FICHIER sauvefichier;	/* sauve les infos sur le fichier courant */

	sauvefichier = *fichier;
	if (first)
	{
		raz_zones(fcherche);
		first = 0;
	}
	if (! *vignorer)
		strcpy(vignorer, "N");

	 /* la touche escape vide les champs de saisie */
	 /* utile si on a remplace une chaine par un espace */
	 /* si l'on ne s'en souvient plus la fois suivante */
	 /* ca peut faire quelque chose d'indesirable */
	 set_esc_clear(ON);

	if (lire_formulaire_simple(fcherche) == OKBUTTON)
	{
		set_refresh(OFF);
		tous = 0;
		fini = 0;
		while ((fichier->courante->suivante != NULL) && (! fini))
		{
			ptr = fichier->courante->texte;
			while (((ptrouve = cherche(ptr, vachercher)) != NULL) && (! fini))
			{
				/* affiche l'ecran et place le curseur sous la chaine trouvee */
				diff = (int)(ptrouve - fichier->courante->texte);
				fichier->premier_caractere = (diff / COLS) * COLS;
				fichier->colonne = (diff % COLS);
				fichier->affiche_ligne(fichier->courante, fichier->ligne, fichier->courante->texte, fichier->premier_caractere);
				affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
				affiche_status_line(fichier, diff + 1);
				move(fichier->ligne, fichier->colonne);
				refresh();
				set_refresh(OFF);

				sauvefichier = *fichier;

				ptr = ptrouve + strlen(vachercher);

				touche = 0; /* evite warning */
				if (! tous)
				{
					touche = question();
					if (touche == 'A')
						tous = 1;
				}

				if (tous || (touche == 'R'))
				{
					/* si pas le test sur *vremplacer et qu'on appuie sur 'T' */
					/* alors toutes les occurences sont supprimees */
					/* or, on n'autorise les suppressions que */
					/* si l'on appuie sur 'R' car sur 'T' */
					/* c'est trop dangereux !!! */
					if ((*vremplacer && tous) || (touche == 'R'))
					{
						ptd = buf;
						pts = fichier->courante->texte;
						while (pts < ptrouve)
							*ptd++ = *pts++;
						strcpy(ptd, vremplacer);
						strcat(ptd, ptr);

						realloue_texte(fichier->courante, buf);
						ptr = fichier->courante->texte + diff + strlen(vremplacer);

						/* reaffiche la ligne modifiee */
						fichier->affiche_ligne(fichier->courante, fichier->ligne, fichier->courante->texte, fichier->premier_caractere);
						refresh(); /* valide reaffichage */

						/* le fichier a ete modifie */
						/* on l'indique pour les 2 car s'il n'y a */
						/* qu'une seule occurence, le fichier n'est pas */
						/* marque comme modifie */
						fichier->saved = sauvefichier.saved = 0;
					}
				}
				else if (touche != 'C')
					fini = 1;
			}
			descendre_ligne(fichier);
		}
	}

	 /* retablit le fonctionnement normal de Escape dans un formulaire */
	 set_esc_clear(OFF);

	*fichier = sauvefichier;
	affiche_ecran(fichier, 0, NULL);
}
/**********************************************************************/
char *cherche(char *ligne, char *motif)
{
	char sauvlig[SZBUF + 2];
	char sauvmotif[80];
	char *ptr;

	strcpy(sauvlig, ligne);
	strcpy(sauvmotif, motif);
	if (*vignorer != 'N')	/* si on doit ignorer les MAJ/min */
	{
		for (ptr = sauvlig; *ptr; ptr++)
			*ptr = (char)toupper((unsigned int)(unsigned char)*ptr);
		for (ptr = sauvmotif; *ptr; ptr++)
			*ptr = (char)toupper((unsigned int)(unsigned char)*ptr);
	}

	ptr = strstr(sauvlig, sauvmotif);
	if (ptr != NULL)
		ptr = ligne + (int)(ptr - sauvlig);
	return(ptr);
}
/**********************************************************************/
void goto_line(FICHIER *fichier, int num)
{
	top_of_file(fichier);	/* on se place en debut de fichier */

	set_refresh(OFF);
	while ((fichier->courante->suivante != NULL) && (fichier->numcourante != num))
		descendre_ligne(fichier);
	affiche_ecran(fichier, 0, NULL);
}
/**********************************************************************/
int execute_commande(FICHIER *fichier)
{
	chtype *buffer;
	char cmd[256];
	int num;
	char *fname;
	static int first = 1;	/* on ne remet la zone a zero qu'au premier passage */

	if (first)
	{
		/* la prochaine fois pas de remise a zero */
		/* pour eviter de taper 2 fois la meme commande */
		raz_zones(fcmdline);
		first = 0;
	}

	if (lire_formulaire_simple(fcmdline) == OKBUTTON)
	{
		/* est-ce une commande interne ? */
		if (*vcmdline == ':')
		{
			if (! strncmp(vcmdline + 1, "macro ", 6))
			{
				/* copie la macro demandee sur la macro par defaut */
				if (*(vcmdline + 7))
				{
					sprintf(cmd, "cp %s %s >/dev/null 2>/dev/null", vcmdline + 7, MACROFILE);
					system(cmd);
				}
			}
			else if (! strncmp(vcmdline + 1, "edit ", 5))
			{
				if (*(vcmdline + 6))
				{
					/* il n'y aura jamais de free() pour le malloc() suivant ! */
					fname = (char *)malloc(strlen(vcmdline + 6) + 1);
					if (fname != NULL)
					{
						strcpy(fname, vcmdline + 6);
						edite_autre_fichier(fname);
						return(FICHIERSUIVANT);
					}
					else
						erreur(memory_error, NULL);
				}
			}
			else if ((! strncmp(vcmdline + 1, "help", 4)) || (! strncmp(vcmdline + 1, "aide", 4)))
			{
				return(escape_help());
			}
			else if (isdigit(*(vcmdline + 1)))
			{
				num = atoi(vcmdline + 1);
				if (num >= 1)
					goto_line(fichier, num);
			}
		}
		else
		{
			/* sinon commande externe */
			buffer = sauve_ecran();
			if (buffer != NULL)
			{
				system(vcmdline);

				restaure_ecran(buffer, ATTENDSTOUCHE);
			}
		}
	}
	return(0);
}
/**********************************************************************/
void appelle_shell(void)
{
	chtype *buffer;
	pid_t pid;
	int status;
	char *shell;
	char *ps1;
	char sauvps1[80];
	char newps1[80];

	buffer = sauve_ecran();
	if (buffer != NULL)
	{
		pid = fork();
		if (pid == -1)
		{
			/* rien ne prouve que cette boite d'alerte fonctionne */
			/* mais rien ne prouve le contraire non plus */
			if (language == ENGLISH)
				erreur("Error while forking", NULL);
			else
				erreur("Erreur durant le fork", NULL);
		}
		else if (! pid)
		{
			if ((shell = (char *)getenv("SHELL")) == NULL)
				shell = "/bin/sh";

			if ((ps1 = (char *)getenv("PS1")) == NULL)
				strcpy(sauvps1, "$");
			else
				strcpy(sauvps1, ps1);

			strcpy(newps1, "PS1=<< je >> ");
			strcat(newps1, sauvps1);
			putenv(newps1);

			execl(shell, shell, NULL);

			/* les lignes suivantes ne sont normalement jamais atteintes */
			/* sauf si l'exec lui meme echoue */
			strcpy(newps1, "PS1=");
			strcat(newps1, sauvps1);
			putenv(newps1);
			if (language == ENGLISH)
				fprintf(stderr, "Error while exec-ing Shell\n");
			else
				fprintf(stderr, "Erreur en lancant le Shell\n");
			exit(0);
		}
		else
			wait(&status);

		restaure_ecran(buffer, NATTENDSPAS);

		if (! WIFEXITED(status))
		{
			if (language == ENGLISH)
				erreur("Error while running Shell", NULL);
			else
				erreur("Erreur durant l'execution du Shell", NULL);
		}
	}
}
/**********************************************************************/
void remonter_page(FICHIER *fichier)
{
	int i;

	set_refresh(OFF);
	for (i = 0; i < (YMAX - YMIN); i++)
		remonter_ligne(fichier);
	set_refresh(ON);
	refresh();
}
/**********************************************************************/
void descendre_page(FICHIER *fichier)
{
	int i;

	set_refresh(OFF);
	for (i = 0; i < (YMAX - YMIN); i++)
		descendre_ligne(fichier);
	set_refresh(ON);
	refresh();
}
/**********************************************************************/
void remonter_ligne(FICHIER *fichier)
{
	LIGNE *nouveau;		/* pour calcul nouveau debut d'ecran */


	if (fichier->courante->precedente != NULL)
	{
		fichier->numcourante--;
		fichier->courante = fichier->courante->precedente;
		nouveau = haut(fichier);
		if (nouveau != fichier->debut_ecran)
		{
			fichier->debut_ecran = nouveau;
			fichier->affiche_ligne(fichier->courante, YMIN, fichier->courante->texte, 0);
		}
	}
}
/**********************************************************************/
void descendre_ligne(FICHIER *fichier)
{
	LIGNE *nouveau;		/* pour calcul nouveau debut d'ecran */


	if (fichier->courante->suivante != NULL)
	{
		fichier->numcourante++;
		fichier->courante = fichier->courante->suivante;
		nouveau = bas(fichier);
		if (nouveau != fichier->debut_ecran)
		{
			fichier->debut_ecran = nouveau;
			fichier->affiche_ligne(fichier->courante, YMAX, fichier->courante->texte, 0);
		}
	}
}
/**********************************************************************/
void insertion_ligne(FICHIER *fichier)
{
	LIGNE *nouvelle;	/* pour calcul nouvelle ligne courante */


	if (fichier->courante->suivante == NULL) /* on est sur la derniere ligne */
		beep();
	else
	{
		fichier->numcourante++;

		fichier->debut_ecran = bas(fichier);

		nouvelle = insere_ligne(fichier, NULL, fichier->courante, fichier->courante->suivante);
		if (nouvelle == NULL)
			beep();
		else
		{
			fichier->courante = nouvelle;
			affiche_ecran(fichier, 0, NULL);
		}
	}
}
/**********************************************************************/
void destruction_ligne(FICHIER *fichier)
{
	LIGNE *nouvelle;	/* pour calcul nouvelle ligne courante */

	/* si premiere ou derniere ligne */
	if ((fichier->courante->precedente == NULL) || (fichier->courante->suivante == NULL))
		beep();
	else
	{
		nouvelle = fichier->courante->suivante;
		if (fichier->debut_ecran == fichier->courante)
		{
			fichier->debut_ecran = nouvelle;
		}
		supprime_ligne(fichier, fichier->courante);
		fichier->courante = nouvelle;
		affiche_ecran(fichier, 0, NULL);
	}
}
/**********************************************************************/
void affiche_ecran(FICHIER *fichier, int decalage, LIGNE *ignore)
{
	int i;		/* compteur */
	LIGNE *debut;	/* pointeur vers la ligne de debut de l'ecran */
	int sauvex;	/* pour sauver l'abscisse */
	int sauvey;	/* pour sauver l'ordonnee */

	set_refresh(OFF);
	getyx(stdscr, sauvey, sauvex);
	for (debut = fichier->debut_ecran, i = YMIN; i <= YMAX; i++)
	{
		move(i, XMIN);
		if (debut->texte != NULL)
		{
			if ((ignore == NULL) || (debut != ignore))
				fichier->affiche_ligne(debut, i, debut->texte, decalage);
		}
		if (debut->suivante == NULL && (i < YMAX))    /* derniere ligne */
		{
			move(i + 1, XMIN);
			clrtobot();
			move(i, XMIN);
			break;
		}
		else
		{
			debut = debut->suivante;
		}
	}
	move(sauvey, sauvex);
	set_refresh(ON);
	refresh();
}
/**********************************************************************/
LIGNE *bas(FICHIER *fichier)
{
	LIGNE *debecr;

	debecr = fichier->debut_ecran;
	if (fichier->ligne < YMAX)
		fichier->ligne++;
	else
	{
		if (debecr->suivante == NULL)
			beep();
		else
		{
			debecr = debecr->suivante;
			fichier->numdebecr++;

			scrollup(YMIN, YMAX);
		}
	}
	return(debecr);
}
/**********************************************************************/
LIGNE *haut(FICHIER *fichier)
{
	LIGNE *debecr;

	debecr = fichier->debut_ecran;
	if (fichier->ligne > YMIN)
		fichier->ligne--;
	else
	{
		if (debecr->precedente == NULL)
			beep();
		else
		{
			debecr = debecr->precedente;
			fichier->numdebecr--;

			scrolldown(YMIN, YMAX);
		}
	}
	return(debecr);
}
/**********************************************************************/
void supprime_ligne(FICHIER *fichier, LIGNE *courante)
{
	if (courante != NULL)
	{
		courante->precedente->suivante = courante->suivante;
		courante->suivante->precedente = courante->precedente;
		if (courante->texte != NULL)
			free(courante->texte);
		free(courante);

		fichier->nblignes--;
	}
}
/**********************************************************************/
int chargement(FILE *fpi, FICHIER *fichier)
{
	int retcode;		/* code de retour */
	LIGNE *courante;	/* ligne courante */
	char *ptr;		/* pointeur dans buftmp */

	retcode = 0;
	fichier->nblignes = 0;
	if (fpi != NULL)
	{
		courante = fichier->premiere;
		while (fgets(buftmp, SZBUF, fpi) != NULL)
		{
			ptr = buftmp + strlen(buftmp) - 1;
			while (ptr >= buftmp)
			{
				if ((*ptr == '\n') || (*ptr == '\r'))
					*ptr-- = '\0';
				else
					break;
			}

			if ((courante = insere_ligne(fichier, buftmp, courante, fichier->derniere)) == NULL)
			{
				retcode = -3;
				break;
			}
		}
	}
	if (! fichier->nblignes)
	{
		if ((courante = insere_ligne(fichier, NULL, fichier->premiere, fichier->derniere)) == NULL)
			retcode = -3;
	}
	return(retcode);
}
/**********************************************************************/
void dechargement(FILE *fpo, FICHIER *fichier)
{
	LIGNE *courante;	/* ptr sur ligne courante */

	for (courante = fichier->premiere->suivante; courante != fichier->derniere; courante = courante->suivante)
	{
		/* ecrit ligne courante dans fichier */
		if (courante->texte != NULL)
			strip(fpo, courante->texte);
	}
}
/**********************************************************************/
int saisir_ligne(FICHIER *fichier)
{
	int quitte;		/* indicateur de sortie */
	int touche;		/* touche lue au clavier */
	int pos;		/* position caractere courant */
	char buffer[SZBUF + 2]; /* buffer de travail */
	char *ptr;		/* pointeur pour comptage d'espaces */
	int nbspaces;		/* nbre d'espaces pour tabulation */
	int i;			/* compteur */
	LIGNE *sauvcourante;
	static int decalage = 0; /* decalage a faire vers la droite */


	if (fichier->courante->texte != NULL)
		strcpy(buffer, fichier->courante->texte);
	else
	{
		memset(buffer, ' ', decalage);
		*(buffer + decalage) = '\0';
	}

	fichier->premier_caractere = 0;

	i = strlen(buffer);
	if (fichier->colonne <= i)
	{
		pos = decalage;
		if (fichier->colonne < pos)
			i = pos;
		else
			i = fichier->colonne;
	}

	/* on se place sur le premier caractere significatif */
	/* trouver quelque chose de mieux pour ce qui depasse l'ecran */
	set_refresh(OFF);
	fichier->colonne = 0;
	while (i > 0)
	{
		droite(fichier, buffer, 0, 0);
		i--;
	}
	fichier->affiche_ligne(fichier->courante, fichier->ligne, buffer, fichier->premier_caractere);
	if ((fichier->colonne + fichier->premier_caractere) > (COLS - 1))
		affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
	set_refresh(ON);
	refresh();

	quitte = 0;
	while (! quitte)
	{
		set_refresh(OFF);
		pos = fichier->colonne + fichier->premier_caractere;
		affiche_status_line(fichier, pos + 1);
		affiche_help_line();
		set_refresh(ON);
		move(fichier->ligne, fichier->colonne);
		refresh();

		touche = fread_key();
		switch (touche)
		{
			case ESCAPE:
				quitte = handle_escape_key();
				break;

			case KEY_F(1):
				quitte = COMMANDE;
				break;

			case KEY_F(2):
				quitte = SHELL;
				break;

			case KEY_F(3):
				if (! fichier->saved)
				{
					if (language == ENGLISH)
					{
						if (confirmation("Do you really want to quit ?"))
							quitte = QUITTE;
					}
					else
					{
						if (confirmation("Voulez-vous vraiment quitter ?"))
							quitte = QUITTE;
					}
				}
				else
					quitte = QUITTE;
				break;

			case KEY_F(4):
				quitte = SAUVE;
				break;

			case KEY_F(5):
				quitte = CHERCHE;
				break;

			case KEY_F(6):
				quitte = MARQUER_BLOC;
				break;

			case KEY_F(7):
				quitte = DEMARQUER_BLOC;
				break;

			case KEY_F(8):
				quitte = COPIER_BLOC;
				break;

			case KEY_F(9):
				quitte = DEPLACER_BLOC;
				break;

			case KEY_F(10):
				quitte = SUPPRIMER_BLOC;
				break;

			case Ctrl_N:
				quitte = FICHIERSUIVANT;
				break;

			case '\r':
			case '\n':
				if (fichier->courante->debut_marque == NOTMARKED)
					quitte = INSERE_LIGNE;
				else
					beep();
				break;

			case Ctrl_Y:
				if (fichier->courante->debut_marque == NOTMARKED)
					quitte = DETRUIT_LIGNE;
				else
					beep();
				break;

			case KEY_UP:
				quitte = REMONTE_LIGNE;
				break;

			case KEY_DOWN:
				quitte = DESCEND_LIGNE;
				break;

			case KEY_PPAGE:
				quitte = REMONTE_PAGE;
				break;

			case KEY_NPAGE:
				quitte = DESCEND_PAGE;
				break;

			case KEY_HOME:
				gauche(fichier, buffer, 0, 1);
				affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
				break;

#ifdef KEY_END
			case KEY_END:
				droite(fichier, buffer, 0, 1);
				if (fichier->colonne + fichier->premier_caractere > (COLS - 1))
					affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
				break;
#endif	/* KEY_END */

			case KEY_LEFT:
				if ((fichier->colonne == XMIN) && fichier->premier_caractere)
					affiche_ecran(fichier, fichier->premier_caractere - 1, fichier->courante);
				gauche(fichier, buffer, 0, 0);
				break;

			case KEY_RIGHT:
				droite(fichier, buffer, 0, 0);
				if (fichier->colonne + fichier->premier_caractere > (COLS - 1))
					affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
				break;

			case KEY_IC:
				inverse_insert_mode();
				break;

			default:
				/* le fichier a ete modifie */
				fichier->saved = 0;

				if ((fichier->courante->precedente != NULL) && (fichier->courante->suivante != NULL))
				{
					switch (touche)
					{
						case Ctrl_W:
							if (fichier->courante->debut_marque == NOTMARKED)
							{
								if (insere_ligne(fichier, &buffer[pos], fichier->courante, fichier->courante->suivante) != NULL)
								{
									buffer[pos] = '\0';
									realloue_texte(fichier->courante, buffer);
									affiche_ecran(fichier, fichier->premier_caractere, NULL);
								}
								else
									erreur(memory_error, NULL);
							}
							break;

						case '\t':
							nbspaces = tabsize - (pos % tabsize);
							if (insert_mode == INSERT)
								inserespace(buffer, pos, nbspaces);
							while (nbspaces--)
							{
								droite(fichier, buffer, 1, 0);
								if (fichier->colonne + fichier->premier_caractere > (COLS - 1))
									affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
							}
							break;

						case Shift_TAB:
						case KEY_BTAB:
							nbspaces = (pos % tabsize);
							if (! nbspaces)
								nbspaces = tabsize;
							while (nbspaces--)
							{
								gauche(fichier, buffer, 1, 0);
								affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
							}
							break;

						case KEY_DC:
							if (buffer[pos])
							{
								strcpy(buffer + pos, buffer + pos + 1);
								fichier->affiche_ligne(fichier->courante, fichier->ligne, buffer, fichier->premier_caractere);
							}
							else
							{
								if ((fichier->courante->debut_marque == NOTMARKED) && (fichier->courante->suivante->debut_marque == NOTMARKED))
								{
									if (fichier->courante->suivante->suivante != NULL)
									{
										strcpy(buffer + pos, fichier->courante->suivante->texte);
										realloue_texte(fichier->courante, buffer);
										sauvcourante = fichier->courante;
										fichier->courante = fichier->courante->suivante;
										destruction_ligne(fichier);
										fichier->courante = sauvcourante;
									}
									else
										beep();
								}
							}
							break;

						case KEY_BACKSPACE:
						case MY_BACKSPACE:
							if (fichier->colonne > XMIN)
							{
								buffer[pos - 1] = '\0';
								strcat(buffer, buffer + pos);
								gauche(fichier, buffer, 1, 0);
								affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
							}
							else
							{
								if (*(buffer + fichier->premier_caractere))
								{
									strcpy(buffer + fichier->premier_caractere, buffer + fichier->premier_caractere + 1);
									fichier->affiche_ligne(fichier->courante, fichier->ligne, buffer, fichier->premier_caractere);
									affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
								}
								else
									beep();
							}
							break;

						default:
							if (touche >= ' ' && touche <= 255)
							{
								if ((insert_mode == REPLACE) && buffer[pos])
								{
									/* correction bug */
									buffer[pos] = (char)touche;
								}
								else
								{
									strcpy(buftmp, buffer + pos);
									*(buffer + pos) = (char)touche;
									*(buffer + pos + 1) = '\0';
									strcat(buffer, buftmp);
								}
								*(buffer + SZBUF) = '\0';

								droite(fichier, buffer, 1, 0);
								if (fichier->colonne + fichier->premier_caractere > (COLS - 1))
									affiche_ecran(fichier, fichier->premier_caractere, fichier->courante);
							}
							break;
					}
				}
				break;
		}
	}

	for (ptr = buffer; *ptr == ' '; ptr++) ;
	decalage = (ptr - buffer);

	if ((fichier->courante->precedente != NULL) && (fichier->courante->suivante != NULL))
		realloue_texte(fichier->courante, buffer);
	fichier->affiche_ligne(fichier->courante, fichier->ligne, fichier->courante->texte, 0);
	affiche_ecran(fichier, 0, fichier->courante);

	return(quitte);
}
/**********************************************************************/
void droite(FICHIER *fichier, char *buf, int affiche, int end)
{
	do
	{
		if ((fichier->colonne < XMAX) && buf[fichier->colonne])
			fichier->colonne++;
		else
		{
			if ((fichier->colonne + fichier->premier_caractere < SZBUF) && (buf[fichier->colonne + fichier->premier_caractere]))
			{
				fichier->premier_caractere++;
				affiche = 1;
			}
			else
			{
				/* si c'est la touche END */
				if (end)
					end = 0;
			}
		}
	}
	while (end) ;

	if (affiche)
		fichier->affiche_ligne(fichier->courante, fichier->ligne, buf, fichier->premier_caractere);
}
/**********************************************************************/
void gauche(FICHIER *fichier, char *buf, int affiche, int home)
{
	do
	{
		if (fichier->colonne > XMIN)
			fichier->colonne--;
		else
		{
			if (fichier->premier_caractere)
			{
				fichier->premier_caractere--;
				affiche = 1;
			}
			else
			{
				/* si c'est la touche HOME */
				if (home)
					home = 0;
			}
		}
	}
	while (home) ;

	if (affiche)
		fichier->affiche_ligne(fichier->courante, fichier->ligne, buf, fichier->premier_caractere);
}
/**********************************************************************/
void expand(char *texte, char *buffer)
{
	int nbspaces;	/* nbre d'espaces pou arriver a une tab */
	char *ptr;	/* pointeur dans ligne courante */
	char *ptw;	/* pointeur dans buffer */

	if (texte != NULL)
	{
		ptw = buffer;
		ptr = texte;
		while (*ptr)
		{
			if (*ptr == '\t')
			{
				nbspaces = tabsize - ((ptw - buffer) % tabsize);
				while (nbspaces--)
					*ptw++ = ' ';
			}
			else
				*ptw++ = *ptr;
			ptr++;
		}
		*ptw = ' ';	/* rentre dans le test */
		while ((ptw >= buffer) && isspace(*ptw))
			*ptw-- = '\0';
	}
	else
		*buffer = '\0';
}
/**********************************************************************/
void strip(FILE *fpo, char *buffer)
{
	char ligne[SZBUF + 2];	/* buffer de travail */
	int guillemet;		/* ne strippe pas les chaines entre guillemets */
	char *ptr;		/* pointeur dans buffer */
	char *ptw;		/* pointeur dans ligne */
	int nbspaces;
	int nbtabs;
	int position;


	ptr = buffer;
	ptw = ligne;
	guillemet = 0;
	position = 1;
	nbtabs = nbspaces = 0;
	for (position = 1; *ptr; ++position)
	{
		if (*ptr == '\"')
			guillemet++;

		if (! (guillemet & 1))
		{
			if (*ptr == ' ')
			{
				if (position % TABSIZE)
					nbspaces++;
				else
				{
					if ((! nbspaces) && (*(ptr + 1) != ' '))
						nbspaces++;
					else
					{
						nbspaces = 0;
						nbtabs++;
					}
				}
			}
			else
			{
				for ( ; nbtabs > 0; --nbtabs)
					*ptw++ = '\t';
				for ( ; nbspaces > 0; --nbspaces)
					*ptw++ = ' ';
				*ptw++ = *ptr;
			}
		}
		else
		{
			for ( ; nbtabs > 0; --nbtabs)
				*ptw++ = '\t';
			for ( ; nbspaces > 0; --nbspaces)
				*ptw++ = ' ';
			*ptw++ = *ptr;
		}
		ptr++;
	}
	*ptw = '\0';

	fprintf(fpo, "%s\n", ligne);
}
/**********************************************************************/
int realloue_texte(LIGNE *plig, char *texte)
{
	char *ptr;		/* pointeur tempo pour alloc */

	if (plig != NULL)
	{
		ptr = (char *)malloc(strlen(texte) + 1);
		if (ptr != NULL)
		{
			if (plig->texte != NULL)
				free(plig->texte);

			plig->texte = ptr;
			strcpy(plig->texte, texte);
			return(0);
		}
		else
			return(-1);
	}
	else
		return(-1);
}
/**********************************************************************/
LIGNE *insere_ligne(FICHIER *fichier, char *buf, LIGNE *avant, LIGNE *apres)
{
	LIGNE *courante;	/* pointeur sur ligne courante */
	char buffer[SZBUF + 2]; /* buffer de travail */

	courante = (LIGNE *)malloc(sizeof(LIGNE));
	if (courante != NULL)
	{
		courante->precedente = avant;
		courante->suivante = apres;
		avant->suivante = courante;
		apres->precedente = courante;

		if (buf != NULL)
		{
			expand(buf, buffer);
			courante->texte = (char *)malloc(strlen(buffer) + 1);
			if (courante->texte != NULL)
			{
				strcpy(courante->texte, buffer);
				courante->debut_marque = courante->fin_marque = NOTMARKED;
				fichier->nblignes++;
			}
			else
			{
				avant->suivante = apres;
				apres->precedente = avant;
				free(courante);
				courante = NULL;
			}
		}
		else
		{
			courante->texte = NULL;
			courante->debut_marque = courante->fin_marque = NOTMARKED;
			fichier->nblignes++;
		}
	}
	return(courante);
}
/**********************************************************************/
char *inserespace(char *string, int pos, int nbre)
{
	char buffer[SZBUF + 2]; /* buffer temporaire */
	char *ptw;		/* pointeur pour ecrire */
	int lg;			/* longueur de la chaine en entree */

	lg = strlen(string);
	if (lg < pos)
		return(NULL);
	else
	{
		strncpy(buffer, string, pos);
		ptw = buffer + pos;
		while (nbre--)
			*ptw++ = ' ';
		strcpy(ptw, string + pos);
		strcpy(string, buffer);

		return(string);
	}
}
/**********************************************************************/
