/**********************************************************************/
/* JE.H : Definition pour l'editeur JE				      */
/**********************************************************************/

/*
    Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    Ce programme fait partie du package JE et est soumis, comme le
    reste du package JE, a la Gnu General Public License version 2
    ou superieure dont voici un extrait et dont vous pouvez lire
    la totalite en consultant le fichier COPYING.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _JE_
#define _JE_

typedef struct _FUNCMAP_
	{
		char *funcname;
		int (*function)(void);
	}
	FUNCMAP;

typedef struct _ESCKEYMAP_
	{
		int esckey;
		char *esckeyname;
		FUNCMAP *func;
	}
	ESCKEYMAP;

typedef struct _LIGNE_
	{
		struct _LIGNE_ *precedente;
		struct _LIGNE_ *suivante;
		int debut_marque;
		int fin_marque;
		char *texte;
	}
	LIGNE;

typedef struct _FICHIER_
	{
		char *filename;
		LIGNE *premiere;
		LIGNE *derniere;
		LIGNE *courante;
		LIGNE *debut_ecran;
		int numcourante;
		int numdebecr;
		int saved;
		int nblignes;
		int first;
		int ligne;
		int colonne;
		int premier_caractere;
		void (*affiche_ligne)(LIGNE *, int, char *, int);
	}
	FICHIER;

#define INSERE_LIGNE	1
#define REMONTE_LIGNE	2
#define DESCEND_LIGNE	3
#define SAUVE		4
#define DETRUIT_LIGNE	5
#define QUITTE		6
#define REMONTE_PAGE	7
#define DESCEND_PAGE	8
#define SHELL		9
#define CHERCHE		10
#define COMMANDE	11
#define MARQUER_BLOC	12
#define DEMARQUER_BLOC	13
#define COPIER_BLOC	14
#define DEPLACER_BLOC	15
#define SUPPRIMER_BLOC	16
#define FICHIERSUIVANT	17

#define ASKCONFIRMATION		1
#define DONTASKCONFIRMATION	0
#define SZBUF			2048
#define XMIN			0
#define YMIN			1
#define NBCOULEURS		25
#define NOTMARKED		-1
#define OUI			1
#define NON			0

#define ATTENDSTOUCHE		1
#define NATTENDSPAS		0
#define INSERT			2
#define REPLACE			1

#define ENGLISH			0
#define FRENCH			1

#define NOMACRO			0
#define RECORDING		1
#define PLAYING			2
#define RECORDED		3
#define MACROFILE		"macro.jem"

#define COULEUR_STATUS		1	/* couleur de la barre d'etat */
#define COULEUR_AIDE		2	/* couleur de la barre d'aide */
#define COULEUR_MOTCLE		3	/* couleur des mots clefs du langage */
#define COULEUR_CHAINE		4	/* couleur des chaines entre guillemets du langage */
#define COULEUR_COMMENT		5	/* couleur des commentaires du langage */
#define COULEUR_ALERTE1		6	/* couleurs des boites d'alerte */
#define COULEUR_ALERTE2		7	/* couleur du bouton selectionne */
#define COULEUR_NORMALE		8	/* couleur du reste */
#define COULEUR_BLOC		9	/* couleur des blocs marques */
#define COULEUR_CHAR	       10	/* couleur des constantes caracteres */

#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <sys/stat.h>
#include <curses.h>

	/* quelques lignes pour un peu plus de portabilite !!! */
#ifndef __NCURSES_H	/* curses.h n'est pas Ncurses.h */
#define __NCURSES_NOT_PRESENT__
#ifndef COLOR_BLACK	/* si les couleurs ne sont pas definies */
#define		COLOR_BLACK		(chtype)A_NORMAL
#define		COLOR_RED		(chtype)A_NORMAL
#define		COLOR_GREEN		(chtype)A_NORMAL
#define		COLOR_YELLOW		(chtype)A_NORMAL
#define		COLOR_BLUE		(chtype)A_NORMAL
#define		COLOR_MAGENTA		(chtype)A_NORMAL
#define		COLOR_CYAN		(chtype)A_NORMAL
#define		COLOR_WHITE		(chtype)A_NORMAL
#define		has_colors()		FALSE
#define		start_color()
#define		init_pair(a, b, c)
#define		COLOR_PAIR(a)		(chtype)A_NORMAL
#endif /* COLOR_BLACK */
#endif /* __NCURSES_H */

#define		min(a, b)	((a < b) ? a : b)
#define		max(a, b)	((a > b) ? a : b)

#define MY_BACKSPACE	127	/* probleme avec mon Linux et ncurses 1.9.4 */
#define TAB		'\t'
#define ENTER		'\r'
#define LINEFEED	'\n'
#define ESPACE		' '
#define ESCAPE		0x001b
#define Ctrl_N		14	/* Next file */
#define Ctrl_P		16	/* Play macro */
#define Ctrl_R		18	/* Record a macro / stop Recording */
#define Ctrl_T		20	/* Shift Tab if not present */
#define Ctrl_W		23	/* Wrap: pour couper une ligne a la position du curseur */
#define Ctrl_Y		25	/* ca, c'est du WordStar */
#define Shift_TAB	Ctrl_T

void supprime_ligne(FICHIER *fichier, LIGNE *courante);
LIGNE *insere_ligne(FICHIER *fichier, char *buf, LIGNE *avant, LIGNE *apres);
int edite_autre_fichier(char *filename);

void debute_ecran(void);
void termine_ecran(void);
int allume_curseur(int type);
void eteint_curseur(void);
void restaure_curseur(void);

int alerte(int bouton, char *titre, char *txtboutons);

int set_refresh(int on_off);
int get_refresh(void);
int get_txt(int x, int y, int xmax, int ymax, chtype *dest);
int put_txt(int x, int y, int xmax, int ymax, chtype *src);

void put_ligne(LIGNE *courante, int y, char *texte, int decalage);
void print_ligne(LIGNE *courante, int y, char *texte, int decalage);
void put_statligne(int x, int y, char *texte, chtype attrib);

void clr_scr(void);

void scrollup(int ymin, int ymax);
void scrolldown(int ymin, int ymax);

int fread_key(void);

int terminal_type(void);

int bloc_dans_ce_fichier(FICHIER *fichier);
void demarquer_bloc(void);
void marquer_bloc(FICHIER *fichier);
void copier_bloc(FICHIER *fichier);
void deplacer_bloc(FICHIER *fichier);
void supprimer_bloc(int ask);

void erreur(char *fmt, char *mess);
void inverse_insert_mode(void);
chtype *sauve_ecran(void);
void restaure_ecran(chtype *buffer, int waitakey);
void affiche_status_line(FICHIER *fichier, int colonne);
void affiche_help_line(void);
int cfamily(char *filename);
int question(void);

chtype hg(void);
chtype hm(void);
chtype hd(void);
chtype mg(void);
chtype mm(void);
chtype md(void);
chtype bg(void);
chtype bm(void);
chtype bd(void);
chtype hgd(void);
chtype hmd(void);
chtype hdd(void);
chtype mgd(void);
chtype mmd(void);
chtype mdd(void);
chtype bgd(void);
chtype bmd(void);
chtype bdd(void);
chtype ht(void);
chtype bt(void);
chtype gt(void);
chtype dt(void);
chtype htd(void);
chtype btd(void);
chtype gtd(void);
chtype dtd(void);

void initialise_profile(char *filename);

int confirmation(char *question);

int handle_escape_key(void);
void build_default_esckeymap(void);
FUNCMAP *findescfunction(char *funcname);
void mapesckeytofunction(char *esckeyname, char *funcname);
int escape_help(void);

#ifndef _JE_TABCOUL_

extern int language;
extern int macro;
extern int XMAX;
extern int YMAX;
extern chtype couleur[NBCOULEURS];

#endif		/* _JE_TABCOUL_ */

#endif		/* _JE_ */
/**********************************************************************/
